package org.apache.bcel.verifier.structurals;

package .tmp;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.apache.bcel.generic.CodeExceptionGen;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.verifier.structurals.ExceptionHandler;

public class ExceptionHandlers {
  private static final ExceptionHandler[] EMPTY_ARRAY = new ExceptionHandler[0];
  
  private final Map<InstructionHandle, Set<ExceptionHandler>> exceptionHandlers = new HashMap<>();
  
  public ExceptionHandlers(MethodGen mg) {
    CodeExceptionGen[] cegs = mg.getExceptionHandlers();
    for (CodeExceptionGen ceg : cegs) {
      ExceptionHandler eh = new ExceptionHandler(ceg.getCatchType(), ceg.getHandlerPC());
      for (InstructionHandle ih = ceg.getStartPC(); ih != ceg.getEndPC().getNext(); ih = ih.getNext()) {
        ((Set<ExceptionHandler>)this.exceptionHandlers.computeIfAbsent(ih, k -> new HashSet())).add(eh);
        ((Set<ExceptionHandler>)this.exceptionHandlers.computeIfAbsent(ih, k -> new HashSet())).add(eh);
      } 
    } 
  }
  
  public ExceptionHandler[] getExceptionHandlers(InstructionHandle ih) {
    Set<ExceptionHandler> hsSet = this.exceptionHandlers.get(ih);
    if (hsSet == null)
      return EMPTY_ARRAY; 
    return hsSet.<ExceptionHandler>toArray(ExceptionHandler.EMPTY_ARRAY);
  }
}
